<?php
/**
 * Blackbird MenuManager Module
 *
 * NOTICE OF LICENSE
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@bird.eu so we can send you a copy immediately.
 *
 * @category            Blackbird
 * @package		Blackbird_MenuManager
 * @copyright           Copyright (c) 2016 Blackbird (http://black.bird.eu)
 * @author		Blackbird Team
 */
namespace Blackbird\MenuManager\Block\NodeType;

use Magento\Backend\Block\Template;
use Magento\Backend\Block\Template\Context;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Profiler;
use Blackbird\MenuManager\Api\NodeTypeInterfaceFront;

class CmsPage extends AbstractNodeTypeFront
{
    protected $nodes;
    protected $pageUrls;
    protected $pageIds;
    /**
     * @var ResourceConnection
     */
    private $connection;

    /**
     * @var Profiler
     */
    private $profiler;

    public function __construct(
        Context $context,
        ResourceConnection $connection,
        Profiler $profiler,
        $data = []
    ) {
        $this->connection = $connection;
        $this->profiler = $profiler;
        parent::__construct($context, $data);
    }


    public function fetchData(array $nodes)
    {
        $this->profiler->start(__METHOD__);
        $localNodes = [];
        $pagesCodes = [];
        foreach ($nodes as $node) {
            $localNodes[$node->getId()] = $node;
            $pagesCodes[] = $node->getContent();
        }
        $this->nodes = $localNodes;

        $pageTable = $this->connection->getTableName('cms_page');
        $storeTable = $this->connection->getTableName('cms_page_store');
        $select = $this->connection->getConnection('read')->select()->from(
            ['p' => $pageTable],
            ['page_id', 'identifier']
        )->join(['s' => $storeTable], 'p.page_id = s.page_id', [])->where(
            's.store_id IN (0, ?)',
            $this->_storeManager->getStore()->getId()
        )->where('p.identifier IN (?)', $pagesCodes)->where('p.is_active = ?', 1)->order('s.store_id ASC');
        $codes = $this->connection->getConnection('read')->fetchAll($select);
        $this->pageIds = [];
        foreach ($codes as $row) {
            $this->pageIds[$row['identifier']] = $row['page_id'];
        }

        $table = $this->connection->getTableName('url_rewrite');
        $select = $this->connection->getConnection('read')
                                   ->select()
                                   ->from($table, ['entity_id', 'request_path'])
                                   ->where('entity_type = ?', 'cms-page')
                                   ->where('store_id = ?', $this->_storeManager->getStore()->getId())
                                   ->where('entity_id IN (?)', array_values($this->pageIds));
        $this->pageUrls = $this->connection->getConnection('read')->fetchPairs($select);
        $this->profiler->stop(__METHOD__);
    }

    /**
     * define the url of the node
     *
     * @param $node
     * @return string
     */
    public function getUrlNode($node)
    {
        if($node->getEntityId()){
            $url = $this->_storeManager->getStore()->getBaseUrl() . $node->getEntityId();
        } else {
            $url = $this->_storeManager->getStore()->getBaseUrl();
        }

        $this->setUrl($url);
        return $url;
    }

    /**
     * @param $nodeId
     * @param $level
     * @param $classes
     * @param $childrenHtml
     * @param $childrenArray
     * @return string
     */
    public function getHtml($nodeId, $level, $classes, $childrenHtml, $childrenArray)
    {
        $node = $this->nodes[$nodeId];

        $node->setUrlPath($this->getUrlNode($node));

        $url = $this->getData('url');
        $classes = $this->getIsActiveClass($url, $node, $classes, $childrenArray);

        return parent::getHtml($nodeId, $level, $classes, $childrenHtml, $childrenArray);
    }
}